PROJECT_ID_ENV_VAR = "MANTIK_PROJECT_ID"
DATA_REPOSITORY_ID_ENV_VAR = "MANTIK_DATA_REPOSITORY_ID"
EXPERIMENT_REPOSITORY_ID_ENV_VAR = "MANTIK_EXPERIMENT_REPOSITORY_ID"
CODE_REPOSITORY_ID_ENV_VAR = "MANTIK_CODE_REPOSITORY_ID"
MODEL_ID_ENV_VAR = "MANTIK_MODEL_ID"
CONNECTION_ID_ENV_VAR = "MANTIK_CONNECTION_ID"
RUN_ID_ENV_VAR = "MANTIK_RUN_ID"
